import sqlite3
import os
import json
import time

from core.core import SYMData, Server, StepType, User
from dataclasses import dataclass, field

@dataclass()
class DBE(object):
    """
    DB handler
    Stores session-recurrent primary keys for quick inserts
    """

    id_experiment : int = field(default=False)
    step_types_sym: int = field(default=False)
    step_types_quest: int = field(default=False)
    steps: list = field(default=False)
    co: sqlite3.Connection = field(default=False, init=False)
    
    def __post_init__(self):
        self.co = sqlite3.connect('sym_server.db')

    def get_step_id(self, nbr: int):
        return [x[1] for x in self.steps if x[0] == nbr][0]



    def insert_quest_data(self, step_nbr: int, blob, user_id: int):
        """
        Injects data into the corresponding questionnaire table
        :param self: 
        :param user_id: id of the user
        :param step_nbr: number of the questionnaire step
        :param blob: multidict data to ingest
        :return:
        """
        cr = self.co.cursor()
        step_id = self.get_step_id(step_nbr)
        insert_data = f"""insert into Questionnaires (answers, _user, _step) 
                          values ("{blob}", {user_id}, {step_id})"""
        cr.execute(insert_data)
        self.co.commit()
        return
    
    
    def insert_sym_data(self, sd: SYMData, step_nbr: int):
        """
    
        :param self:
        :param sd: SYMData to be stored
        :param step_nbr: number of the SYM Step (in case of multiple ones)
        :return:
        """
        cr = self.co.cursor()
        step_id = self.get_step_id(step_nbr)
        insert_data = f"""insert into Sym_data (x, y, _user, timestamp, _step)
                          values ({sd.x}, {sd.y}, {sd.user_id}, "{sd.timestamp}", {step_id})"""
        cr.execute(insert_data)
        self.co.commit()
        return
    
    
    def insert_user_id(self, user: User):
        """
    
        :param self:
        :param user:
        :return:
        """
        cr = self.co.cursor()
        pref_user = f"""insert into User (id_user, "_experiment") 
                        values ({user.user_id}, {self.id_experiment});"""
        cr.execute(pref_user)
        self.co.commit()
        return
    
    
    def declare_experiment(self, server: Server):
        """
        Create the experiment in db along with Questionnaires
        :param self: DBE (see object declaration)
        :param server: server object
        :return:
        """
        pref_exp = f"""insert into Experiment (name, description)
                       values ('{server.name}', '{server.description}')"""
        cr = self.co.cursor()
        cr.execute(pref_exp)
        self.co.commit()
        res = cr.execute(f"""select id_experiment 
                       from Experiment 
                       where name == '{server.name}' 
                       and description == '{server.description}'""")
        self.id_experiment = exp_id = res.fetchone()[0]
        self.steps = []
    
        # Populate step types
        res = cr.execute("""select id_s_type from Step_type where s_type == 'SYM'""")
        self.step_types_sym = res.fetchone()[0]
    
        res = cr.execute("""select id_s_type from Step_type where s_type == 'Questionnaire'""")
        self.step_types_quest = res.fetchone()[0]
    
        # populate steps
        for step in server.scenario:
            step_type_id = None
            declare_blob = None
            if step.step_type is StepType.SYM:
                step_type_id = self.step_types_sym
                pass
            elif step.step_type is StepType.Questionnaire:
                step_type_id = self.step_types_quest
                declare_blob = json.load(open('quest_templates/' + step.template_path))
            else:
                continue
            pref_step = f"""insert into Steps (step_type, step_index, declaration, _experiment)
                            values ({step_type_id},
                            {step.step_rank},
                            '{declare_blob}',
                            {exp_id})"""
            self.co.commit()
            print(f"""_experiment == {self.id_experiment} and step_index == {step.step_rank}""")
            res = cr.execute(f"""select id_step from Steps 
                                 where _experiment == {self.id_experiment} 
                                 and step_index == {step.step_rank}""")
            self.steps.append([step.step_rank, res.fetchone()[0]])
            cr.execute(pref_step)
            self.co.commit()
        return
    
    
    def verify_db(self):
        """
        Verifies the presence of basic tables
        # TODO could be more complete
        :return:
        """
        cr = self.co.cursor()
        # check tables names
        table_names = [
            'Step_type',
            'User',
            'Questionnaires',
            'Experiment',
            'Steps',
            'Sym_data'
        ]
        res = cr.execute("""select name from sqlite_master""")
        for name in table_names:
            if name not in res.fetchall():
                raise AssertionError
        return
    
    
    def create_blank(self):
        """
        Creates a new database set
        :return: connection and cursor objects for created db
        """
        self.co.close()
        os.remove('../sym_server.db')
        self.co = sqlite3.connect('sym_server.db')
        cr = self.co.cursor()
    
        table_step_type = """
        create table main.Step_type
        (
            s_type    TEXT    not null,
            id_s_type integer not null
                constraint Step_type_pk
                    primary key autoincrement
        );"""
        populate_step_type = """
        insert into Step_type (s_type) values ('SYM');
        insert into Step_type (s_type) values ('Questionnaire');
        """
        table_experiment = """
        create table main.Experiment
        (
            id_experiment integer not null
                constraint Experiment_pk
                    primary key autoincrement,
            name          text,
            description   text
        );
        
        """
        table_user = """
        create table main.User
        (
            id_user     integer not null
                constraint User_pk
                    primary key,
            _experiment integer not null
                constraint User_Experiment_id_experiment_fk
                    references main.Experiment
        );"""
        table_steps = """
        create table main.Steps
        (
            id_step     integer              not null
                constraint Steps_pk
                    primary key autoincrement,
            step_type   integer              not null
                constraint Steps_Step_type_id_s_type_fk
                    references main.Step_type,
            step_index  integer default NULL not null,
            declaration BLOB    default '',
            _experiment integer              not null
                constraint Steps_Experiment_id_experiment_fk
                    references main.Experiment
        );"""
        table_questionnaires = """
        create table main.Questionnaires
        (
            id_questionnaire integer not null
                constraint Questionnaires_pk
                    primary key autoincrement,
            answers          BLOB,
            _user            integer not null
                constraint Questionnaires_User_id_user_fk
                    references main.User,
            _step            integer not null
                constraint Questionnaires_Steps_id_step_fk
                    references main.Steps
        );"""
        table_sym_data = """
        create table main.Sym_data
        (
            id_sym_data integer not null
                constraint Sym_data_pk
                    primary key autoincrement,
            x           integer not null,
            y           integer not null,
            _user       integer not null
                constraint Sym_data_User_id_user_fk
                    references main.User,
            timestamp   integer not null,
            _step       integer not null
                constraint Sym_data_Steps_id_step_fk
                    references main.Steps
        );"""
    
        cr.execute(table_step_type)
        self.co.commit()
        cr.execute(populate_step_type)
        self.co.commit()
        cr.execute(table_experiment)
        self.co.commit()
        cr.execute(table_user)
        self.co.commit()
        cr.execute(table_steps)
        self.co.commit()
        cr.execute(table_questionnaires)
        self.co.commit()
        cr.execute(table_sym_data)
        self.co.commit()
        return


if __name__ == '__main__':
    pass

